#include "../lib/Headers/mm_malloc.h"
#include <new>

void *__aligned_new(std::size_t size, 
                    std::size_t align) throw (std::bad_alloc)
{
  if (size == 0)
    size = 1;

  void* p = 0;

  while ((p = _mm_malloc(size, align)) == 0) {
    // If _mm_malloc fails and there is a new_handler,
    // call it to try free up memory.
    std::new_handler nh = std::set_new_handler(0);
    if (nh) {
      std::set_new_handler(nh);
      nh();
    }
    else
      throw std::bad_alloc();
  }
  return p;
}

void *__aligned_new(std::size_t size, 
                    std::size_t align, 
                    const std::nothrow_t&) throw()
{
  void* p = 0;

  try {
    p = __aligned_new(size, align);
  }
  catch (...) {}

  return p;
}

void *__aligned_new_arr(std::size_t size, 
                        std::size_t align) throw (std::bad_alloc)
{
  return __aligned_new(size, align);
}

void *__aligned_new_arr(std::size_t size, 
                        std::size_t align, 
                        const std::nothrow_t& nothrow) throw()
{
  void* p = 0;
  try {
    p = __aligned_new_arr(size, align);
  }
  catch (...) {}

  return p;
}

// Alligned analogs of standard library operator delete functions declared 
// in the <new> header
void __aligned_delete(void* ptr) throw ()
{
  if (ptr)
    _mm_free(ptr);
}

void __aligned_delete(void* ptr, const std::nothrow_t&) throw ()
{
  __aligned_delete(ptr);
}

void __aligned_delete_arr(void* ptr) throw ()
{
  __aligned_delete(ptr);
}

void __aligned_delete_arr(void* ptr, const std::nothrow_t&) throw ()
{
  __aligned_delete_arr(ptr);
}
